@@ -13,21 +13,23 @@ import json
 import zipfile
 import itertools as it
 
-from ._export import StreamFormatter
+from ._export import StreamFormatter, ContinuousEntityFormatter
 
 
-class EuPMCFormatter(StreamFormatter):
+class EuPMCFormatter(StreamFormatter, ContinuousEntityFormatter):
     """
     Formatter for Europe PMC's named-entity annotation format.
     """
 
     ext = 'jsonl'
 
-    def __init__(self, provider, src='MED', info=('type', 'pref', 'uri')):
+    def __init__(self, provider, src='MED', meta=('type', 'pref', 'uri'),
+                 **kwargs):
+        super().__init__(**kwargs)
         self.metadata = {'provider': provider, 'src': src}
         (self.type,
          self.pref,
-         self.uri) = info
+         self.uri) = meta
 
     def write(self, content, stream):
         documents = content.units('document')
@@ -46,13 +48,12 @@ class EuPMCFormatter(StreamFormatter):
             text = sent.text
             offset = sent.start
             section = self._section_name(sent.section, meta['src'])
-            locations = it.groupby(sent.entities,
+            locations = it.groupby(self.iter_entities(sent),
                                    key=lambda e: (e.start-offset, e.end-offset))
             for l, ((start, end), colocated) in enumerate(locations, start=1):
-                types = it.groupby(colocated, key=lambda e: e.info[self.type])
+                types = it.groupby(colocated, key=self._entity_type)
                 for type_, entities in types:
-                    entities = set((e.info[self.pref], e.info[self.uri])
-                                   for e in entities)
+                    entities = set(map(self._entity_name_uri, entities))
                     ann = {
                         'position': '{}.{}'.format(s, l),
                         'prefix': text[max(start-20, 0):start],
@@ -65,6 +66,24 @@ class EuPMCFormatter(StreamFormatter):
                     doc['anns'].append(ann)
         return doc
 
+    def _entity_type(self, entity):
+        return self._entity_meta(entity, self.type)
+
+    def _entity_name_uri(self, entity):
+        return tuple(self._entity_meta(entity, k) for k in (self.pref, self.uri))
+
+    @staticmethod
+    def _entity_meta(entity, key):
+        try:
+            return entity.metadata[key]
+        except KeyError as e:
+            if e.args == (key,):
+                raise ValueError(
+                    'Need entity attribute: {!r} not found in Entity.metadata. '
+                    'Please check the `meta` option.'
+                    .format(key))
+            raise
+
     def _section_name(self, section, src):
         name = section.type
         if src == 'PMC':
